package com.demo.projecttest;

import org.eclipse.ui.IStartup;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;

public class StartupExtension implements IStartup {
	
	static final IWorkspace workspace = ResourcesPlugin.getWorkspace();
	@Override
	public void earlyStartup() {
		
		workspace.addResourceChangeListener(rcl);
		
		Job job = new Job("Create Project") {

			@Override
			protected IStatus run(IProgressMonitor arg0) {
				
				IProject project = ResourcesPlugin.getWorkspace().getRoot().getProject("DemoProject1");
				createUpdateProject(project);
				
				project = ResourcesPlugin.getWorkspace().getRoot().getProject("DemoProject2");
				createUpdateProject(project);
				
				project = ResourcesPlugin.getWorkspace().getRoot().getProject("DemoProject3");
				createUpdateProject(project);
				
				return Status.OK_STATUS;
			}

			private void createUpdateProject(IProject project) {
				if ( project.exists()) {
					
					System.out.println("Project " + project.getName() + " exists");
				} 
				else {
					final IPath path = new Path(project.getName());
					final IPath projectLocation = Platform.getLocation().append(path);
					
					copyTemplate(project);
					IProjectDescription pDesc = ResourcesPlugin.getWorkspace().newProjectDescription(project.getName());
					pDesc.setLocation(projectLocation);

					try {
						project.create(pDesc, null);
						
					} catch (CoreException e) {
						e.printStackTrace();
					}
				}
			}

			
		};
		job.setSystem(false);
		job.setPriority(Job.INTERACTIVE);
		job.setRule(ResourcesPlugin.getWorkspace().getRoot());
		job.schedule();
		
	}
	

	private void copyTemplate(IProject projectHandle) {
		File sourceTemplate = new File("E:\\Workspaces\\eclipse-2019-09\\com.demo.projecttest\\.demo-project");
		File destination = new File(ResourcesPlugin.getWorkspace().getRoot().getLocationURI().getPath(), projectHandle.getName());
		
		if ( !destination.exists() ) {
			destination.mkdir();
		}
		if (sourceTemplate.exists()) {
			File destTemplate = new File(destination, ".project");
			
			try {
				copy(sourceTemplate, destTemplate);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	public static void copy(File src, File dest) throws IOException {
        InputStream is = null;
        OutputStream os = null;
        try {
            is = new FileInputStream(src);
            os = new FileOutputStream(dest);

            byte[] buf = new byte[1024];

            int bytesRead;
            while ((bytesRead = is.read(buf)) > 0) {
                os.write(buf, 0, bytesRead);
            }
        } finally {
            is.close();
            os.close();
        }
    }
	
	static IResourceChangeListener rcl = new IResourceChangeListener() {

		@Override
		public void resourceChanged(IResourceChangeEvent event) {
			try {
				if (event == null || event.getDelta() == null)
					return;
				event.getDelta().accept(new IResourceDeltaVisitor() {
					
					@Override
					public boolean visit(IResourceDelta delta) throws CoreException {
					
						IResource resource = delta.getResource();
						if (((resource.getType() & IResource.PROJECT) != 0) && resource.getProject().isOpen()
								&& delta.getKind() == IResourceDelta.CHANGED
								&& ((delta.getFlags() & IResourceDelta.OPEN) != 0)) {

							final IProject project = (IProject) resource;
							IProjectDescription description = project.getDescription();
							
							String[] natures = description.getNatureIds();
							System.out.println("NatureIds -> " + project + " " + Arrays.asList(natures));
						}
						return true;
					}
				});
			} catch (CoreException e) {
				e.printStackTrace();
			}
		}
		
	};
}
